<?php

class RestRequest {
 








 function send($path, $data = null, $params = null, $method = null) {
 $stream = $this->newStream($method, $data);
 $url = $this->getApiUrl($path, $params);
 $body = $stream->sendRequest($url);
 if ($stream->getError()) {
 return $this->handleError($url, $stream);
 }
 return $this->handleResponse($body, $stream);
 }

 function newStream($method = null, $data = null) {
 if (is_null($method)) {
 $method = $this->getDefaultMethod();
 }

 $stream = new FetchStream($method);
 foreach ($this->getRequestHeaders() as $key => $value) {
 $stream->setHeader($key, $value);
 }

 if ( ! is_null($data) ) {
 $stream->setPostData($data);
 }

 return $stream;
 }

 function getRequestHeaders() {
 return array("Content-Type" => "text/html; charset=utf8");
 }

 function getDefaultQueryParams() {
 global $SESSION_TOKEN;
 return array("sid" => $SESSION_TOKEN);
 }

 function getApiUrl($path, $params = null) {
 global $CN_LOCAL_SERVER, $PORTLETAPP_PREFIX_INTERNAL;
 if ( ! is_null($params) ) {
 $params = array_merge($this->getDefaultQueryParams(), $params);
 } else {
 $params = $this->getDefaultQueryParams();
 }
 $url = $CN_LOCAL_SERVER . $PORTLETAPP_PREFIX_INTERNAL . 'rest' . $path . "?" . http_build_query($params);
 return $url;
 }

 function handleFatalError($message) {
 halt(7, "REST request aborted with an error: `".$message."'");
 }

 



 function handleError($url, $stream) {
 if ($this->isRecoverable($stream)) {
 return null;
 }
 return $this->handleFatalError("($url) ".$stream->getError());
 }

 









 function getDefaultMethod() {
 return "GET";
 }

 











 function isRecoverable($stream) {
 return 404 == $stream->getReturnCode();
 }

 


 function isOK($stream) {
 return 200 == $stream->getReturnCode();
 }

 




 function handleResponse($body, $stream) {
 if ($this->isOK($stream)) {
 return $this->handleOK($body, $stream);
 } else {
 return $this->handleNotOK($body, $stream);
 }
 }

 




 function handleNotOK($body, $stream) {
 return $this->handleFatalError(
 "unexpected status code (".$stream->getReturnCode().")"
 . " error (".$stream->getError().")");
 }

 



 
}

class RestResponse {
 var $code;
 var $body;

 function RestResponse($code, $body) {
 $this->code = $code;
 $this->body = $body;
 }

 function getBody() {
 return $this->body;
 }

 function getCode() {
 return $this->code;
 }
}




class ValidationRequest extends RestRequest {
 



 function getDefaultMethod() {
 return "POST";
 }

 



 function isRecoverable($stream) {
 return false;
 }

 




 function getDefaultQueryParams() {
 return array_merge(RestRequest::getDefaultQueryParams(),
 array("formattedError" => "true"));
 }

 


 function handleOK($body, $stream) {
 return new ValidationResponse($stream->getReturnCode(), $body);
 }
}







class ShortcutValidationRequest extends ValidationRequest {
 


 function send($path, $data = null, $params = null, $method = null) {
 global $VALIDATION;
 if (isset($VALIDATION['shortcutRegex'])) {
 if (preg_match($VALIDATION['shortcutRegex'], $data)) {
 return $this->handleShortcut($path, $data, $params);
 }
 }
 return ValidationRequest::send($path, $data, $params, $method);
 }

 




 function handleShortcut($path, $data, $params) {
 return new ValidationResponse(200, $data, false);
 }
}

class ValidationResponse extends RestResponse {
 var $decoded;

 






 function ValidationResponse($code, $body, $json = true) {
 
 $this->RestResponse($code, $body);
 if ($json) {
 $this->decoded = json_decode($body, true);
 } else {
 $this->decoded = array(
 "messages" => array(),
 "cleanMarkup" => $body,
 "formattedError" => ""
 );
 }
 }

 function getMessages() {
 return $this->decoded["messages"];
 }

 function getCleanMarkup() {
 return $this->decoded["cleanMarkup"];
 }
 
 



 function getFormattedError() {
 return $this->decoded["formattedError"];
 }

 function isError() {
 foreach ($this->getMessages() as $message) {
 if ($message["fatal"]) {
 return true;
 }
 }
 return false;
 }
}




class TagPartValidationRequest extends ShortcutValidationRequest {
 function validateTagPart($unsafe, $part_id, $params = null) {
 $part_id = (int)$part_id;
 return $this->send("/validate/tagPart/$part_id", $unsafe, $params);
 }
}




class GenericValidationRequest extends ShortcutValidationRequest {
 var $suffix = "";

 



 function send($path, $data = null, $params = null, $method = null) {
 return ShortcutValidationRequest::send($path . $this->suffix, $data, $params, $method);
 }

 



 

 






 function setPathSuffix($suffix) {
 if ( ! is_null($suffix) ) {
 $suffix = "/".$suffix;
 }
 $this->suffix = $suffix;
 }
}





class NodeInputValidationRequest extends GenericValidationRequest {
 function validateNodeInput($unsafe, $node_id = null, $params = null) {
 global $NODE_ID;
 if (is_null($node_id)) {
 $node_id = $NODE_ID;

 
 if ( ! $node_id ) {
 warning(7, "No NODE_ID available for NodeInputValidationRequest,"
 . " falling back to InputValidationRequest");
 $requ = new InputValidationRequest();
 $requ->setPathSuffix("genericInput");
 return $requ->validate($unsafe, $params);
 }
 }
 $node_id = (int)$node_id;
 return $this->send("/validate/nodeInput/$node_id", $unsafe, $params);
 }

 




 function validate($unsafe, $params = null) {
 return $this->validateNodeInput($unsafe, null, $params);
 }
}




class InputValidationRequest extends GenericValidationRequest {
 


 function validate($unsafe, $params = null) {
 return $this->send("/validate", $unsafe, $params);
 }
}




class PolicyGroupRequest extends RestRequest {

 function getForPartType($type_id) {
 global $POLICY_CACHE, $POLICY_GROUP_CACHE;
 if (isset($POLICY_GROUP_CACHE[$type_id])) {
 return $POLICY_GROUP_CACHE[$type_id];
 }

 $type_id = (int)$type_id;
 $group = $this->send("/policyMap/partType/$type_id/policyGroup");

 $POLICY_GROUP_CACHE[$type_id] = $group;
 foreach ($group->getPolicies() as $policy) {
 $POLICY_CACHE[$policy["uri"]] = $policy;
 }

 return $group;
 }

 


 function handleOK($body, $stream) {
 return new PolicyGroupResponse($stream->getReturnCode(), $body);
 }
}

class PolicyGroupResponse extends RestResponse {
 var $decoded;

 function PolicyGroupResponse($code, $body) {
 $this->RestResponse($code, $body);
 $this->decoded = json_decode($body, true);
 }

 function getPolicies() {
 return $this->decoded["policies"];
 }

 function getPolicyByIndex($idx) {
 $policies = $this->getPolicies();
 return $policies[$idx];
 }

 function getIndexForPolicy($policy) {
 $policies = $this->getPolicies();
 for ($i = 0; $i < count($policies); $i++) {
 if ($policies[$i]["uri"] == $policy["uri"]) {
 return $i;
 }
 }
 return null;
 }

 function getPolicyByUri($uri) {
 foreach ($this->getPolicies() as $policy) {
 if ($policy["uri"] == $uri) {
 return $policy;
 }
 }
 return null;
 }

 function getDefaultPolicy() {
 foreach ($this->getPolicies() as $policy) {
 if ($policy["default"]) {
 return $policy;
 }
 }
 halt(7, "each policy group must have a default policy");
 }
}




class PolicyRequest extends RestRequest {

 




 function getForUri($uri) {
 global $POLICY_CACHE;
 if (isset($POLICY_CACHE[$uri])) {
 return $POLICY_CACHE[$uri];
 }

 $policy = $this->send("/policyMap/policy", null, array("uri" => $uri));

 $POLICY_CACHE[$policy["uri"]] = $policy;
 return $policy;
 }

 


 function handleOK($body, $stream) {
 return json_decode($body, true);
 }

 



 function handleNotOK($body, $stream) {
 return null;
 }
}













function policy_name_for_part_type($type_id, $uri = null) {
 $suffix = "";
 if (strlen($uri)) {
 $requ = new PolicyRequest();
 $policy = $requ->getForUri($uri);
 
 
 if (is_null($policy)) {
 $suffix = " (" . $uri . ")";
 }
 }
 if ( ! isset($policy) ) {
 $requ = new PolicyGroupRequest();
 $group = $requ->getForPartType($type_id);
 $policy = $group->getDefaultPolicy();
 }
 return $policy["name"] . $suffix;
}






function global_validation_error_message() {
 global $VALIDATION_RESULT;
 return $VALIDATION_RESULT->getFormattedError();
}






function validation_error_js_alert($result) {
 $text = "";
 foreach ($result->getMessages() as $message) {
 if ($message["fatal"]) {
 if ($text) {
 $text .= " ";
 }
 
 $msg = strip_tags($message["message"]);
 $msg = html_entity_decode($msg, ENT_QUOTES);
 
 $msg = trim($msg);
 $text .= $msg;
 }
 }
 return "alert('".addslashes($text)."');";
}









function global_validation_error_js_alert() {
 global $VALIDATION_RESULT;
 if (is_global_validation_error()) {
 return validation_error_js_alert($VALIDATION_RESULT);
 } else {
 return null;
 }
}












function is_global_validation_error() {
 global $VALIDATION_RESULT;
 return ! is_null($VALIDATION_RESULT) && $VALIDATION_RESULT->isError();
}








function set_global_validation_result($result) {
 global $VALIDATION_RESULT;
 $VALIDATION_RESULT = $result;
}





function get_global_validation_result() {
 global $VALIDATION_RESULT;
 return $VALIDATION_RESULT;
}







function reset_global_validation_result() {
 set_global_validation_result(null);
}













function validate_input($suffix, $unsafe, $requ) {
 global $FEATURE;
 if ($FEATURE["validation"]) {
 $requ->setPathSuffix($suffix);
 return $requ->validate($unsafe);
 } else {
 return new ValidationResponse(200, $unsafe, false);
 }
}










function validated_input_value($suffix, $unsafe, $requ) {
 $resp = validate_input($suffix, $unsafe, $requ);
 return $resp->isError() ? null : $resp->getCleanMarkup();
}

















function validate_properties($properties, $requ) {
 $result = array();
 $idx = 0;
 foreach ($properties as $prop_name => $prop_value) {
 $validated = validated_input_value($prop_name, $prop_value, $requ);
 $result[$idx++] = $validated;
 $result[$prop_name] = $validated;
 }
 return $result;
}

















function validate_properties_htmlform($properties, $requ, $html_form = null, $html_form_prop_names = null) {
 $result = validate_properties($properties, $requ);
 $error = false;
 foreach ($result as $prop_name => $prop_value) {
 if ( ! is_numeric($prop_name) && is_null($prop_value) ) {
 $html_form_key = is_null($html_form_prop_names)
 ? $prop_name
 : $html_form_prop_names[$prop_name];
 $html_form->check[$html_form_key]
 = i18n('validation.invalid.' . strtolower($prop_name));
 $error = true;
 }
 }
 return $error ? false : $result;
}




function validate_properties_htmlform3($properties, $requ, $html_form = null, $html_form_prop_names = null) {
 $result = validate_properties($properties, $requ);
 $error = false;
 foreach ($result as $prop_name => $prop_value) {
 if ( ! is_numeric($prop_name) && is_null($prop_value) ) {
 $html_form_key = is_null($html_form_prop_names)
 ? $prop_name
 : $html_form_prop_names[$prop_name];
 $exception = new NodeException(i18n('validation.invalid.'.strtolower($prop_name)));
 $component = $html_form->resolveComponent($html_form_key);
 $component->addError($exception);
 $error = true;
 }
 }
 return $error ? false : $result;

}


